import { Injectable, NotFoundException, StreamableFile } from '@nestjs/common';
import { DbService } from 'apps/backend/src/app/db/db.service';
import { FilesTable } from 'apps/backend/src/app/db/schema';
import { StorageService } from 'apps/backend/src/app/storage/storage.service';
import { eq, ilike } from 'drizzle-orm';


@Injectable()
export class FilesService {
	constructor(
		private readonly storage: StorageService,
		private readonly database: DbService,
	) {}

  /**
	 * Retrieves a file and its related information from the database and associated storage.
	 *
	 * @param fileId The unique identifier for the file to be retrieved.
	 * @return A promise that resolves to an object containing the streamable file and additional file information.
	 * @throws NotFoundException if the file does not exist in the database.
	 */
	public async get(fileId: string): Promise<StreamableFile> {
		const foundFiles = await this.database
			.use()
			.select()
			.from(FilesTable)
			.where(eq(FilesTable.uuid, fileId))
			.prepare("getFileFromId")
			.execute();

		if (foundFiles.length === 0)
			throw new NotFoundException("File not found", {
        description: `Identifier : ${fileId}`
      });

		if (foundFiles.length > 1)
			console.log(
				"Multiples entries found in the database.\nInformation from the first row will be used.",
			);
		const file = foundFiles[0];

		const fileBuffer = await this.storage.read(
			file.checksum,
			file.extension,
			file.isDocumentation,
		);
		const fileInformation = await this.storage.generateInformation(
			fileBuffer,
			file.fileName,
			file.isDocumentation,
		);
		const fileNameWithoutSpaces = file.fileName.replace(/\s/g, "_");
		return new StreamableFile(fileBuffer, {
			disposition: `attachment; filename="${fileNameWithoutSpaces}.${fileInformation.fileType.ext.toLowerCase()}"`
		});
	}

	public async search(limit: number, offset: number, searchField: string) {
		return await this.database
			.use()
			.select()
			.from(FilesTable)
			.where(ilike(FilesTable.fileName, String(searchField)))
			.limit(limit)
			.offset(offset)
			.prepare("searchFiles")
			.execute();
	}

	//TODO save a file
	public async save() {

	}
}
